# Copyright 2012-2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.


DOC_EVENTS = {
    'doc-breadcrumbs': '.%s.%s',
    'doc-title': '.%s.%s',
    'doc-description': '.%s.%s',
    'doc-synopsis-start': '.%s.%s',
    'doc-synopsis-option': '.%s.%s.%s',
    'doc-synopsis-end': '.%s.%s',
    'doc-options-start': '.%s.%s',
    'doc-option': '.%s.%s.%s',
    'doc-option-example': '.%s.%s.%s',
    'doc-options-end': '.%s.%s',
    'doc-examples': '.%s.%s',
    'doc-output': '.%s.%s',
    'doc-subitems-start': '.%s.%s',
    'doc-subitem': '.%s.%s.%s',
    'doc-subitems-end': '.%s.%s',
    }


def fire_event(session, event_name, *fmtargs, **kwargs):
    event = session.create_event(event_name, *fmtargs)
    session.emit(event, **kwargs)


def generate_events(session, help_command):
    # First, register all events
    for event_name in DOC_EVENTS:
        session.register_event(event_name,
                               DOC_EVENTS[event_name])
    # Now generate the documentation events
    fire_event(session, 'doc-breadcrumbs', help_command.event_class,
               help_command.name, help_command=help_command)
    fire_event(session, 'doc-title', help_command.event_class,
               help_command.name, help_command=help_command)
    fire_event(session, 'doc-description', help_command.event_class,
               help_command.name, help_command=help_command)
    fire_event(session, 'doc-synopsis-start', help_command.event_class,
               help_command.name, help_command=help_command)
    if help_command.arg_table:
        for arg_name in help_command.arg_table:
            fire_event(session, 'doc-synopsis-option',
                       help_command.event_class,
                       help_command.name, arg_name,
                       arg_name=arg_name, help_command=help_command)
    fire_event(session, 'doc-synopsis-end', help_command.event_class,
               help_command.name, help_command=help_command)
    fire_event(session, 'doc-options-start', help_command.event_class,
               help_command.name, help_command=help_command)
    if help_command.arg_table:
        for arg_name in help_command.arg_table:
            fire_event(session, 'doc-option', help_command.event_class,
                       help_command.name, arg_name,
                       arg_name=arg_name, help_command=help_command)
            fire_event(session, 'doc-option-example',
                       help_command.event_class,
                       help_command.name, arg_name,
                       arg_name=arg_name, help_command=help_command)
    fire_event(session, 'doc-options-end', help_command.event_class,
               help_command.name, help_command=help_command)
    fire_event(session, 'doc-subitems-start', help_command.event_class,
               help_command.name, help_command=help_command)
    if help_command.command_table:
        for command_name in sorted(help_command.command_table.keys()):
            if hasattr(help_command.command_table[command_name],
                       '_UNDOCUMENTED'):
                continue
            fire_event(session, 'doc-subitem', help_command.event_class,
                       help_command.name, command_name,
                       command_name=command_name, help_command=help_command)
    fire_event(session, 'doc-subitems-end', help_command.event_class,
               help_command.name, help_command=help_command)
    fire_event(session, 'doc-examples', help_command.event_class,
               help_command.name, help_command=help_command)
    fire_event(session, 'doc-output', help_command.event_class,
               help_command.name, help_command=help_command)



#!/usr/bin/env python

"""
distutils/setuptools install script.
"""

try:
    from setuptools import setup
    setup
except ImportError:
    from distutils.core import setup

packages = [
    'bcdoc',
]

requires = ['six>=1.1.0',
            'docutils>=0.10']


setup(
    name='bcdoc',
    version='0.12.0',
    description='ReST document generation tools for botocore.',
    long_description=open('README.rst').read(),
    author='Mitch Garnaat',
    author_email='mitch@garnaat.com',
    url='https://github.com/botocore/bcdoc',
    packages=packages,
    package_dir={'bcdoc': 'bcdoc'},
    install_requires=requires,
    license=open("LICENSE.txt").read(),
    classifiers=(
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.0',
        'Programming Language :: Python :: 3.1',
        'Programming Language :: Python :: 3.2',
    ),
)



# Copyright 2012-2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import logging
from bcdoc.docstringparser import DocStringParser
from bcdoc.style import ReSTStyle

LOG = logging.getLogger('bcdocs')


class ReSTDocument(object):

    def __init__(self, target='man'):
        self.style = ReSTStyle(self)
        self.target = target
        self.parser = DocStringParser(self)
        self.keep_data = True
        self.do_translation = False
        self.translation_map = {}
        self.hrefs = {}
        self._writes = []
        self._last_doc_string = None

    def _write(self, s):
        if self.keep_data and s is not None:
            self._writes.append(s)

    def write(self, content):
        """
        Write content into the document.
        """
        self._write(content)

    def writeln(self, content):
        """
        Write content on a newline.
        """
        self._write('%s%s\n' % (self.style.spaces(), content))

    def peek_write(self):
        """
        Returns the last content written to the document without
        removing it from the stack.
        """
        return self._writes[-1]

    def pop_write(self):
        """
        Removes and returns the last content written to the stack.
        """
        return self._writes.pop()

    def push_write(self, s):
        """
        Places new content on the stack.
        """
        self._writes.append(s)

    def getvalue(self):
        """
        Returns the current content of the document as a string.
        """
        if self.hrefs:
            self.style.new_paragraph()
            for refname, link in self.hrefs.items():
                self.style.link_target_definition(refname, link)
        return ''.join(self._writes).encode('utf-8')

    def translate_words(self, words):
        return [self.translation_map.get(w, w) for w in words]

    def handle_data(self, data):
        if data and self.keep_data:
            self._write(data)

    def include_doc_string(self, doc_string):
        if doc_string:
            try:
                start = len(self._writes)
                self.parser.feed(doc_string)
                end = len(self._writes)
                self._last_doc_string = (start, end)
            except Exception:
                LOG.debug('Error parsing doc string', exc_info=True)
                LOG.debug(doc_string)

    def remove_last_doc_string(self):
        # Removes all writes inserted by last doc string
        if self._last_doc_string is not None:
            start, end = self._last_doc_string
            del self._writes[start:end]
